{ config, lib, pkgs, user, theme, ... }:

{
  home.packages = with pkgs.unstable; [
    xclip

    # language servers
    lua-language-server
    rnix-lsp
    terraform-ls
    tflint
    nodejs
    nodePackages.bash-language-server
    nodePackages.yaml-language-server
    nodePackages.prettier
    nodePackages.eslint_d
    nodePackages.typescript-language-server
    nodePackages.typescript
    nodePackages.dockerfile-language-server-nodejs
    nodePackages.vscode-langservers-extracted
    nodePackages.volar
    lldb
    vscode-extensions.vadimcn.vscode-lldb
    marksman
    clang-tools
    cmake-language-server
  ];

  # symlink for a stable path 
  home.file.".vscode-extensions/vscode-lldb".source = pkgs.unstable.vscode-extensions.vadimcn.vscode-lldb;

  home.sessionVariables = {
    EDITOR = "nvim";
  };

  programs = {
    neovim = {
      enable = true;
      package = pkgs.unstable.neovim-unwrapped;
      plugins = with pkgs.unstable.vimPlugins; [
        # theme
        nvim-solarized-lua

        delimitMate
        vim-bbye
        ansible-vim

        # ui 
        dressing-nvim

        # session handling
        project-nvim
        auto-session

        # escape handling 
        better-escape-nvim

        nvim-treesitter.withAllGrammars
        markdown-preview-nvim # use({ "iamcco/markdown-preview.nvim", run = ":call mkdp#util#install()" })
        toggleterm-nvim
        nvim-notify
        comment-nvim
        lualine-nvim
        tabline-nvim
        indent-blankline-nvim
        plenary-nvim
        vim-illuminate
        registers-nvim
        vim-surround

        editorconfig-nvim

        # git
        gitsigns-nvim
        vim-fugitive
        vim-flog
        git-blame-nvim
        nvim-web-devicons # used by diffview-nvim

        # snippets
        friendly-snippets
        luasnip

        # auto-completion
        nvim-cmp
        cmp-nvim-lsp
        cmp-buffer
        cmp-path
        cmp-cmdline
        cmp-nvim-lua
        cmp_luasnip
        cmp-calc
        cmp-nvim-lsp-signature-help

        # telescope
        telescope-nvim
        telescope-fzf-native-nvim
        telescope-ui-select-nvim

        # LSP
        nvim-lspconfig
        lsp-status-nvim
        rustaceanvim
        neodev-nvim
        null-ls-nvim
        lsp_lines-nvim
        lsp-inlayhints-nvim

        # debugging
        nvim-dap
        nvim-dap-ui
      ];

      extraConfig = ''
        lua << EOF
            vim.o.guifont = "DejaVu Sans Mono:h10"

            if vim.g.neovide then
                vim.g.neovide_fullscreen = true
                vim.g.neovide_cursor_animation_length = 0.01
                vim.g.neovide_cursor_trail_length = 0.01
            end

            ${builtins.readFile ./keymaps.lua }
            ${builtins.readFile ./options.lua }
            ${builtins.readFile ./commands.lua }
            ${builtins.readFile ./treesitter.lua }
            ${builtins.readFile ./lspconfig.lua }
            ${builtins.readFile ./auto-session.lua }
            ${builtins.readFile ./toggleterm.lua }
            ${builtins.readFile ./notify.lua }
            ${builtins.readFile ./comment.lua }
            ${builtins.readFile ./lualine.lua }
            ${builtins.readFile ./tabline.lua }
            ${builtins.readFile ./indent-blankline.lua }
            ${builtins.readFile ./gitsigns.lua }
            ${builtins.readFile ./luasnip.lua }
            ${builtins.readFile ./nvim-cmp.lua }
            ${builtins.readFile ./telescope.lua }

            ${builtins.readFile ./themes.lua }
            vim.opt.background = '${theme}'

            ${builtins.readFile ./project-nvim.lua }
            ${builtins.readFile ./dap.lua }

            vim.g.gitblame_date_format = "%r"

            -- setup illuminate highlight groups
            vim.api.nvim_set_hl(0, "IlluminatedWordText", { bold = true })
            vim.api.nvim_set_hl(0, "IlluminatedWordRead", { link = "IlluminatedWordText" })
            vim.api.nvim_set_hl(0, "IlluminatedWordWrite", { link = "IlluminatedWordText" })
        EOF
      '';
    };
  };
}


